<?php declare(strict_types=1);

namespace Everyware\ProjectPlugin\Components\Adapters;

use Everyware\ProjectPlugin\Wordpress\Contracts\EveryboardWidget;
use Everyware\ProjectPlugin\Wordpress\Contracts\WpWidget;
use Everyware\ProjectPlugin\Components\Contracts\Admin;
use Everyware\ProjectPlugin\Components\Contracts\WidgetComponent;
use Infomaker\Everyware\Support\Str;
use WP_Widget;

class WidgetAdapter extends WP_Widget implements WpWidget, EveryboardWidget
{

    /**
     * @var Admin
     */
    private $component;

    public function __construct(Admin $component, array $widgetOptions = [], array $controlOptions = [])
    {
        $this->component = $component;
        $widgetId = $this->getWidgetId();

        $widgetOptions = array_replace_recursive([
            'classname' => $widgetId,
            'description' => $component->getDescription()
        ], $widgetOptions);

        parent::__construct($widgetId, $component->getName(), $widgetOptions, $controlOptions);
    }

    public function form($instance): ?string
    {
        $this->renderForm($this->updateData($instance));

        return null;
    }

    public function update($new_instance, $old_instance)
    {
        if ($this->saveSettings($new_instance, $old_instance)) {
            return $this->component->getSettings();
        }

        return false;
    }

    public function widget($args, $instance): void
    {
        if ($this->component instanceof WidgetComponent) {
            echo $this->component->generateWidget($this->updateData($instance), $args);
        }
    }

    public function getWidgetId()
    {
        return Str::replaceFirst('widget-', '', $this->component->getInputPrefix());
    }

    /**
     * Function to render widget content in EveryBoard.
     *
     * @param $data
     *
     * @return void
     */
    public function widget_board($data): void
    {
        if ($this->component instanceof WidgetComponent) {
            print $this->component->getBoardContent($data);
        }
    }

    private function renderForm($storedData): void
    {
        echo empty($storedData) ? $this->component->create($storedData) : $this->component->edit($storedData);
    }

    private function saveSettings($newData, $storedData): bool
    {
        if (empty($storedData)) {
            return $this->component->store($newData, $storedData);
        }

        return $this->component->update($newData, $storedData);
    }

    private function updateData($storedData)
    {
        if ($this->component instanceof WidgetComponent) {
            $this->component->setWidgetInstanceNumber($this->number);
        }

        $this->saveSettings($storedData, []);

        return $this->component->getSettings();
    }
}
