<?php declare(strict_types=1);

namespace Everyware\ProjectPlugin\Components;

use Everyware\ProjectPlugin\Components\Contracts\InfoManager;
use Everyware\ProjectPlugin\Components\Contracts\SettingsForm;
use Everyware\ProjectPlugin\Html\FormBuilder;
use Everyware\ProjectPlugin\Html\HtmlBuilder;

/**
 * Class SettingsForm
 * @package Everyware\ProjectPlugin\Components
 */
abstract class ComponentSettingsForm implements SettingsForm
{
    public static $infoMap = [
        'description' => 'Description',
        'name' => 'Plugin Name',
        'version' => 'Version',
    ];

    /**
     * @var SettingsHandler
     */
    protected $settingsHandler;

    /**
     * @var InfoManager
     */
    protected $infoManager;

    public function __construct(InfoManager $infoManager, SettingsHandler $settingsHandler)
    {
        $this->settingsHandler = $settingsHandler;
        $this->infoManager = $infoManager;
    }

    public function currentVersion(): string
    {
        return $this->infoManager->getHeader(static::$infoMap['version']);
    }

    public function getComponentInfo(array $componentInfoMap): array
    {
        return $this->infoManager->extractHeaders($componentInfoMap);
    }

    public function getDescription(): string
    {
        return $this->infoManager->getHeader(static::$infoMap['description']);
    }

    public function getName(): string
    {
        return $this->infoManager->getHeader(static::$infoMap['name']);
    }

    public function create(array $storedData): string
    {
        return $this->formContent($storedData);
    }

    public function edit(array $storedData): string
    {
        return $this->formContent($storedData);
    }

    public function getFormPrefix(): string
    {
        return $this->settingsHandler->getFormPrefix();
    }

    /**
     * @return FormBuilder
     */
    protected function generateFormBuilder(): FormBuilder
    {
        return new FormBuilder(new HtmlBuilder());
    }

    protected function getFormSettings(array $settings = []): array
    {
        return array_replace([
            'name' => $this->getName(),
            'description' => $this->getDescription(),
            'version' => $this->currentVersion(),
            'class_prefix' => $this->getFormPrefix() . '__form',
            'id_prefix' => $this->getFormPrefix()
        ], $settings);
    }

    abstract protected function formContent($storedData):string;
}
