<?php declare(strict_types=1);

namespace Everyware\ProjectPlugin\Components;

class SettingsHandler
{
    /**
     * @var string
     */
    private $formPrefix;

    /**
     * @var string
     */
    protected $currentGroup;

    public function __construct($formPrefix)
    {
        $this->formPrefix = $formPrefix;
    }

    public function generateFormData(array $storedData = []): array
    {
        return $this->setupFields($storedData);
    }

    public function generateGroupedFormData($group, $storedData = [])
    {
        $this->currentGroup = "[$group]";

        $formData = $this->generateFormData($storedData);

        $this->currentGroup = null;

        return $formData;
    }

    public function getFormPrefix(): string
    {
        return $this->formPrefix;
    }

    // Private Functions
    // ======================================================


    private function getFieldMultiValues(string $field, array $instance = []): array
    {
        $returnVal = [];
        foreach ($instance as $i => $value) {

            if ( ! \is_array($value)) {
                return $this->getFieldValues("{$field}[]", $instance);
            }

            foreach ((array)$value as $settingsName => $settingsValue) {
                $returnVal[$i][$settingsName] = $this->getFieldValues("{$field}[{$i}][{$settingsName}]",
                    $instance[$i][$settingsName]);
            }
        }

        return $returnVal;
    }

    private function getFieldValues(string $field, $value): array
    {
        $fieldObj = new SettingsField($field, $value, $this->formPrefix . $this->currentGroup ?? '');

        return $fieldObj->toArray();
    }

    private function setupFields(array $fields = []): array
    {
        $fieldsValues = [];

        foreach ($fields as $field => $value) {
            $fieldsValues[$field] = \is_array($value) ?
                $this->getFieldMultiValues($field, $value) :
                $this->getFieldValues($field, $value);
        }

        return $fieldsValues;
    }
}
