<?php declare(strict_types=1);

namespace Everyware\ProjectPlugin\Components;

use Everyware\ProjectPlugin\Components\Adapters\WidgetAdapter;

/**
 * Class WidgetManager
 * @package Everyware\ProjectPlugin\Components
 */
class WidgetManager
{
    protected $widgets = [];
    /**
     * @var bool
     */
    private $widgetsAdded = false;

    /**
     * @var self
     */
    private static $instance;

    private function __construct()
    {
        add_action('widgets_init', [$this, 'registerAll']);
    }

    public function registerAll(): void
    {
        if ( ! $this->widgetsAdded) {

            foreach ($this->widgets as $widget) {
                $resolver = $widget['resolver'];
                $component = $resolver();

                register_widget(new WidgetAdapter($component, $widget['widget_options'], $widget['control_options']));
            }
        }

        $this->widgetsAdded = true;
    }

    public function addWidget(Callable $resolver, $widgetOptions, $controlOptions): void
    {
        $this->widgets[] = [
            'control_options' => $controlOptions,
            'widget_options' => $widgetOptions,
            'resolver' => $resolver
        ];
    }

    protected static function manager(): WidgetManager
    {
        if( static::$instance === null ) {
            static::$instance = new static();
        }

        return static::$instance;
    }

    public static function register(Callable $resolver, array $widgetOptions = [], array $controlOptions = []): void
    {
        static::manager()->addWidget($resolver, $widgetOptions, $controlOptions);
    }
}
