<?php

namespace Spec\Everyware\ProjectPlugin\Components\Adapters;

use Everyware\ProjectPlugin\Components\Contracts\Admin;
use Everyware\ProjectPlugin\Components\Contracts\WidgetComponent;
use Everyware\ProjectPlugin\Components\Adapters\WidgetAdapter;
use Everyware\ProjectPlugin\Wordpress\Contracts\EveryboardWidget;
use Everyware\ProjectPlugin\Wordpress\Contracts\WpWidget;
use Spec\ComponentAdapterSpec;
use WP_Widget;

class WidgetAdapterSpec extends ComponentAdapterSpec
{
    function let(Admin $componentAdmin)
    {
        $componentAdmin->beADoubleOf(WidgetComponent::class);

        $componentAdmin
            ->getInputPrefix()
            ->shouldBeCalled()
            ->willReturn('widget-ew-test-widget-prefix');

        $componentAdmin
            ->getDescription()
            ->shouldBeCalled()
            ->willReturn($this->success);

        $componentAdmin
            ->getName()
            ->shouldBeCalled()
            ->willReturn($this->success);

        $this->beConstructedWith($componentAdmin);

        $this->shouldImplement(WpWidget::class);
        $this->shouldImplement(EveryboardWidget::class);
        $this->shouldBeAnInstanceOf(WP_Widget::class);
    }

    function it_is_initializable()
    {
        $this->shouldHaveType(WidgetAdapter::class);
    }

    function it_routes_widget_form_to_create_on_creation(Admin $componentAdmin)
    {
        $result = 'ok';
        $storedSettings = [];

        $componentAdmin
            ->setWidgetInstanceNumber(false)
            ->shouldBeCalled();

        $componentAdmin
            ->store($storedSettings, [])
            ->shouldBeCalled()
            ->willReturn(true);

        $componentAdmin
            ->getSettings()
            ->shouldBeCalled()
            ->willReturn($storedSettings);

        $componentAdmin
            ->create($storedSettings)
            ->shouldBeCalled()
            ->willReturn($result);

        $this->form($storedSettings)->shouldReturn(null);
    }

    function it_routes_to_component_edit_if_stored_data_is_found(Admin $componentAdmin)
    {
        $result = 'ok';
        $storedSettings = ['has_settings'];
        $componentAdmin
            ->store($storedSettings, [])
            ->shouldBeCalled()
            ->willReturn(true);

        $componentAdmin
            ->getSettings()
            ->shouldBeCalled()
            ->willReturn($storedSettings);

        $componentAdmin
            ->edit($storedSettings)
            ->shouldBeCalled()
            ->willReturn($result);

        $componentAdmin
            ->setWidgetInstanceNumber(false)
            ->shouldBeCalled();

        $this->form($storedSettings)->shouldReturn(null);
    }

    function it_routes_to_component_store(Admin $componentAdmin)
    {
        $storedSettings = [];
        $newSettings = ['data'];
        $componentAdmin
            ->store($newSettings, $storedSettings)
            ->shouldBeCalled()
            ->willReturn(true);

        $componentAdmin
            ->getSettings()
            ->shouldBeCalled()
            ->willReturn($newSettings);

        $this->update($newSettings, $storedSettings)->shouldReturn($newSettings);
    }

    function it_routes_to_component_update(Admin $componentAdmin)
    {
        $storedSettings = ['has_settings'];
        $newSettings = ['data'];
        $componentAdmin
            ->update($newSettings, $storedSettings)
            ->shouldBeCalled()
            ->willReturn(true);

        $componentAdmin
            ->getSettings()
            ->shouldBeCalled()
            ->willReturn($newSettings);

        $this->update($newSettings, $storedSettings)->shouldReturn($newSettings);
    }

    function it_wont_update_invalid_settings(Admin $componentAdmin)
    {
        $storedSettings = ['has_settings'];
        $newSettings = ['data'];
        $componentAdmin
            ->update($newSettings, $storedSettings)
            ->shouldBeCalled()
            ->willReturn(false);

        $componentAdmin
            ->getSettings()
            ->shouldNotBeCalled();

        $this->update($newSettings, $storedSettings)->shouldReturn(false);
    }

    function it_wont_store_invalid_settings(Admin $componentAdmin)
    {
        $storedSettings = [];
        $newSettings = ['data'];

        $componentAdmin
            ->store($newSettings, $storedSettings)
            ->shouldBeCalled()
            ->willReturn(false);

        $componentAdmin
            ->getSettings()
            ->shouldNotBeCalled();

        $this->update($newSettings, $storedSettings)->shouldReturn(false);
    }

    function it_can_generate_widget_output(Admin $componentAdmin)
    {
        $args = ['data'];
        $storedSettings = ['has_settings'];

        $componentAdmin
            ->setWidgetInstanceNumber(false)
            ->shouldBeCalled();

        $componentAdmin
            ->store($storedSettings, [])
            ->shouldBeCalled()
            ->willReturn(true);

        $componentAdmin
            ->getSettings()
            ->shouldBeCalled()
            ->willReturn($storedSettings);

        $componentAdmin
            ->generateWidget($storedSettings, $args)
            ->shouldBeCalled()
            ->willReturn('ok');

        $this->widget($args, $storedSettings)->shouldReturn(null);
    }

    function it_offers_output_for_when_in_everyBoard(WidgetComponent $componentAdmin)
    {
        $storedSettings = ['has_settings'];

        $componentAdmin
            ->getBoardContent($storedSettings)
            ->shouldBeCalled()
            ->willReturn('ok');

        $this->widget_board($storedSettings)->shouldReturn(null);
    }

    function it_can_create_widget_id_to_match_form_handler_input_prefix(Admin $componentAdmin)
    {
        $componentAdmin
            ->getInputPrefix()
            ->shouldBeCalled()
            ->willReturn('widget-ew-test-widget-prefix');

        $this->getWidgetId()->shouldReturn('ew-test-widget-prefix');
    }
}
