<?php

namespace Spec\Everyware\ProjectPlugin\Components;

use Everyware\ProjectPlugin\Components\SettingsHandler;
use PhpSpec\ObjectBehavior;
use Prophecy\Argument;

class SettingsHandlerSpec extends ObjectBehavior
{
    private $formPrefix = 'field_prefix';
    private $fields = [
        'headline' => '',
        'has_headline' => false,
        'links' => []
    ];

    public function let()
    {
        $this->beConstructedWith($this->formPrefix);
    }

    function it_is_initializable()
    {
        $this->shouldHaveType(SettingsHandler::class);
    }

    public function it_should_generate_proper_form_data()
    {
        $this->generateFormData($this->fields)->shouldReturn([
            'headline' => [
                'id' => $this->formPrefix . '-headline',
                'name' => $this->formPrefix . '[headline]',
                'value' => ''
            ],
            'has_headline' => [
                'id' => $this->formPrefix . '-has_headline',
                'name' => $this->formPrefix . '[has_headline]',
                'value' => false
            ],
            'links' => [],
        ]);
    }

    public function it_supports_overriding_of_fields_uppon_getting_data()
    {
        $this->generateFormData([
            'headline' => 'new Headline',
            'has_headline' => true
        ])->shouldReturn([
            'headline' => [
                'id' => $this->formPrefix . '-headline',
                'name' => $this->formPrefix . '[headline]',
                'value' => 'new Headline'
            ],
            'has_headline' => [
                'id' => $this->formPrefix . '-has_headline',
                'name' => $this->formPrefix . '[has_headline]',
                'value' => true
            ]
        ]);
    }

    public function it_has_support_for_list_fields()
    {
        $this->generateFormData([
            'links' => ['first_link']
        ])->shouldReturn([
            'links' => [
                'id' => $this->formPrefix . '-links',
                'name' => $this->formPrefix . '[links][]',
                'value' => ['first_link']
            ],
        ]);
    }

    public function it_supports_field_objects()
    {
        $this->generateFormData([
            'links' => [
                ['title' => 'first_link', 'url' => 'http://example.com'],
                ['title' => 'second_link', 'url' => 'http://example2.com', 'extra_value' => 'unique']
            ]
        ])->shouldReturn([
            'links' => [
                [
                    'title' => [
                        'id' => $this->formPrefix . '-links-0-title',
                        'name' => $this->formPrefix . '[links][0][title]',
                        'value' => 'first_link',
                    ],
                    'url' => [
                        'id' => $this->formPrefix . '-links-0-url',
                        'name' => $this->formPrefix . '[links][0][url]',
                        'value' => 'http://example.com',
                    ]
                ],
                [
                    'title' => [
                        'id' => $this->formPrefix . '-links-1-title',
                        'name' => $this->formPrefix . '[links][1][title]',
                        'value' => 'second_link',
                    ],
                    'url' => [
                        'id' => $this->formPrefix . '-links-1-url',
                        'name' => $this->formPrefix . '[links][1][url]',
                        'value' => 'http://example2.com',
                    ],
                    'extra_value' => [
                        'id' => $this->formPrefix . '-links-1-extra_value',
                        'name' => $this->formPrefix . '[links][1][extra_value]',
                        'value' => 'unique',
                    ]
                ]
            ],
        ]);
    }

    function it_offers_getter_for_formprefix()
    {
        $this->getFormPrefix()->shouldReturn($this->formPrefix);
    }

    function it_offers_grouping_off_fields()
    {
        $group = 'group';
        $idPrefix = "{$this->formPrefix}-{$group}";
        $namePrefix = "{$this->formPrefix}[{$group}]";
        $this->generateGroupedFormData($group, $this->fields)->shouldReturn([
            'headline' => [
                'id' => $idPrefix . '-headline',
                'name' => $namePrefix . '[headline]',
                'value' => ''
            ],
            'has_headline' => [
                'id' => $idPrefix . '-has_headline',
                'name' => $namePrefix . '[has_headline]',
                'value' => false
            ],
            'links' => []
        ]);
    }
}
